__title__ = 'slim'
__version__ = '0.7.3'
__build__ = 0x000012
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2013-2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'

from slim.helpers import *
from slim.models import *
from slim.models.fields import *
